
/*******************************************************************************
//   Code for application report SLAA501 - "Implementing a Thermocouple Interface
//   with ADC12_A."
//
//   Damian Szmulewicz
//   Texas Instruments, Inc
//   July 2011
//   Built with IAR Embedded Workbench Version: 5.10.4 
********************************************************************************
// THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
// REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY,
// INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR
// COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE.
// TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET
// POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY
// INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR
// YOUR USE OF THE PROGRAM.
//
// IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
// CONS#defineENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY
// THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT
// OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM.
// EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF
// REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS
// OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF
// USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S
// AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF
// YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS
// (U.S.$500).
//
// Unless otherwise stated, the Program written and copyrighted
// by Texas Instruments is distributed as "freeware".  You may,
// only under TI's copyright in the Program, use and modify the
// Program without any charge or restriction.  You may
// distribute to third parties, provided that you transfer a
// copy of this license to the third party and the third party
// agrees to these terms by its first use of the Program. You
// must reproduce the copyright notice and any other legend of
// ownership on each copy or partial copy, of the Program.
//
// You acknowledge and agree that the Program contains
// copyrighted material, trade secrets and other TI proprietary
// information and is protected by copyright laws,
// international copyright treaties, and trade secret laws, as
// well as other intellectual property laws.  To protect TI's
// rights in the Program, you agree not to decompile, reverse
// engineer, disassemble or otherwise translate any object code
// versions of the Program to a human-readable form.  You agree
// that in no event will you alter, remove or destroy any
// copyright notice included in the Program.  TI reserves all
// rights not specifically granted under this license. Except
// as specifically provided herein, nothing in this agreement
// shall be construed as conferring by implication, estoppel,
// or otherwise, upon you, any license or other right under any
// TI patents, copyrights or trade secrets.
//
// You may not use the Program in non-TI devices.
//
*******************************************************************************/

#include   "msp430F5529.h"

 // Status Flags
#define   Num_of_Results   32
#define   Shifter           5 // Value used to efficiently divide and get an average. 
                              // Should be chosen based on the value of Num_of_Results
                              // For Num_of_Results = 2, Shifter = 1; for Num_of_Results = 4
                              // Shifter = 4, etc. 
volatile unsigned int A0results[Num_of_Results];
volatile unsigned int A1results[Num_of_Results];



// Table for TR values 
const unsigned int TR_Temps[]={147,151,155,159,163,167,173,179,185,191,197,204,
                               211,218,225,231,239,247,255,263,272,282,292,302,
                               312,324,336,348,360,372,385,399,413,427,441,455,
                               472,489,506,523,540,561,582,603,624,644,668,692,
                               716,740,767,796,825,854,883,911,945,979,1013,1047,
                               1080,1118,1156,1194,1232,1272,1315,1358,1401,1444,
                               1488,1536,1584,1632,1680,1727,1780,1833,1886,1939,
                               1993};
// Table for TC values
const unsigned int TC_Temps[]={0,32,65,98,131,164,197,230,263,296,329,362,395,
                               428,461,494,527,560,593,626,659,692,725,758,791,
                               824,857,890,923,956,989,1022,1055,1088,1121,1154,
                               1187,1220,1253,1286,1319,1352,1385,1418,1451,1484,
                               1517,1550,1583,1616,1649,1682,1715,1748,1781,1814,
                               1847,1880,1913,1946,1979,2012,2045,2078,2111,2144,
                               2177,2210,2243,2276,2309,2342,2375,2408,2441,2474,
                               2507,2540,2573,2606,2639,2672,2705,2738,2771,2804,
                               2837,2870,2903,2936,2969,3002,3035,3068,3101,3134,
                               3167,3200,3233,3266,3299};

#define NumberOfTRTemps (80)              // Number of elements in TR_Temps
#define NumberOfTemps (101)               // Number of elements in TC_Temps

// Status Flags
#define Too_Low      (0x04)               // Temperature too low flag
#define Too_High     (0x08)               // Temperature too high flag
       
/*******************************************************************************
// RAM variables
*******************************************************************************/

int  A0_TCave;               // Thermocouple ADC reading - average of 32 samples
int  A1_TRave;               // Thermistor ADC reading - average of 32 samples
int signed TR_Temp_Whole;    // Thermistor temperature value
float TR_Tenth;              // Tenths part of TR temperature, x10
int TC_Temp;                 // TC temperature 
int *Flags;                  // Status flags
int Error = 0;               // TC or TR out of range variable

/*******************************************************************************
// Pointers to Flash variables
*******************************************************************************/
int *Off0_flash;             // Offset calibration value for channel 0 (TC)
int *Off1_flash;             // Offset calibration value for channel 1 (TR)
int *Amp_Off_flash;          // Offset calibration value for difference amp

/*******************************************************************************
// Functions
********************************************************************************/

/*------------------------------------------------------------------------------
Sample; This routine takes 32 simultaneous samples of channel 0, channel 1,
      ; channel 2 and channel 3
      ; The samples are stored in respective buffers
------------------------------------------------------------------------------*/
void Sample (void)
{

  ADC12IE = 0x02;          // Enable ADC12IFG.3
  ADC12CTL0 |= ADC12ENC;   // Enable Conversion
  ADC12CTL0 |= ADC12SC;    // Start Conversion
  __bis_SR_register(LPM3_bits+GIE); //Enter LPM3 with interrupts

}
/*------------------------------------------------------------------------------
Average: This Function calculates the average of each ADC channel and stores it
         memory.
------------------------------------------------------------------------------*/
void Average (void)
{
 unsigned long temp_sum0 = 0;  // Temperary sum for channel 0
 unsigned long temp_sum1 = 0;  // Temporary sum for channel 1

  int i=31;
  while(i)
  {
    temp_sum0 = temp_sum0 + A0results[i]; // Add all 32 values CH0
    temp_sum1 = temp_sum1 + A1results[i]; // Add all 32 values CH1
    i--;
  }
// After for loop first value needs to be added to temp_sumx
    temp_sum0 = temp_sum0 + A0results[0];  
    temp_sum1 = temp_sum1 + A1results[0]; 
// Get average and store in memory    
    A0_TCave = temp_sum0 >> Shifter;     //Divide to get average 
    A1_TRave = temp_sum1 >> Shifter;     //Divide to get average
// Substract ADC and OpAmp offset 
    A0_TCave = A0_TCave - *Off0_flash -*Amp_Off_flash;  
    A1_TRave = A1_TRave - *Off1_flash;   
}
/*------------------------------------------------------------------------------
Get_TR_Temp       ; Determine temperature of thermistor
;                 ; using tables and interpolation
------------------------------------------------------------------------------*/
void Get_TR_Temp (void)
{  
     if (A1_TRave < TR_Temps[0]) //To low
   {
       *Flags = Too_Low;
            
   }
    else if (A1_TRave > TR_Temps[NumberOfTRTemps-1]) //To high
   {
       *Flags = Too_High;
   }
   else 
   {  TR_Temp_Whole = 60;   //Assume Max temp of TR
   for(int i=1;NumberOfTemps; i++)  //Lookup table
   {
    if (A1_TRave >= TR_Temps[i])
     {TR_Temp_Whole--;
     }
    else {
      TR_Tenth = TR_Temps[i]-A1_TRave;  //Get thenth part of TR temperature
      TR_Tenth = TR_Tenth /(TR_Temps[i]-TR_Temps[i-1]);
      TR_Tenth = 1-TR_Tenth ;
      TR_Tenth = (int)(TR_Tenth*10);     //Cast to get one sig fig. 
      break;
    }
   }
   }
}
/*------------------------------------------------------------------------------
Get_TC_Temp       ; Determine temperature of thermocouple
                  ; using tables and interpolation
                  ; Compensate for cold-junction temperature retulting in an 
                  ; absolute TC temperature.
------------------------------------------------------------------------------*/
void Get_TC_Temp (void)
{ 
 
  // Channel 0, TC:  
   if (A0_TCave < TC_Temps[0]) //To low
   {
       *Flags = Too_Low;
      
   }
    else if (A0_TCave > TC_Temps[NumberOfTemps-1]) //To high
   {
       *Flags = Too_High;
   }
   else 
   { TC_Temp = 0;                   // Assume TC temperature = 0 to start
   for(int i=1;NumberOfTemps; i++)  //Lookup table
   {
    if (A0_TCave  >= TC_Temps[i])
     TC_Temp++;                     //Increment TC Temp until correct value
    else 
    {
      TC_Temp = TC_Temp+TR_Temp_Whole; // Cold Junction compensation
      break;
    }
   }} 
}


void main(void)
{ int i;
            WDTCTL = WDTPW+WDTHOLD,&WDTCTL;  // Stop WDT
 //Set I/Os as outputs to save power
            P1DIR = 0xFF;
            P1OUT = 0; 
            P2DIR = 0xFF;
            P2OUT = 0; 
            P3DIR = 0xFF;
            P3OUT = 0; 
            P4DIR = 0xFF;
            P4OUT = 0; 
            P5DIR = 0xFF;
            P5OUT = 0; 
            P7DIR = 0xFF;
            P7OUT = 0; 
            P8DIR = 0xFF;
            P8OUT = 0; 
            P6DIR = 0xFF;
            P6OUT = 0; 
            
           
            P6SEL |= BIT1+BIT0;              // Set IOs for ADC (A0 = TC, A1 = TR)
       

          
//Initialize pointers to Flash
            
            Off0_flash = (int *)0x1980;
            Off1_flash = (int *)0x1982; 
            Amp_Off_flash = (int *)0x1984;   

// Set ADC and Opamp Offset here    
            
            *Off0_flash = 0;
            *Off1_flash = 0;
            *Amp_Off_flash = 0;            

            
/*------------------------------------------------------------------------------
            Set Timer A to trigger an interrupt at 1Hz. Source: VLO
------------------------------------------------------------------------------*/
            
             UCSCTL4 |= SELA_1; // ACLK sourced from VLO
             TA0CCTL0 = CCIE;                          // CCR0 interrupt enabled
             TA0CTL |= TASSEL_1+TACLR+MC_0+ID_3;  // Input CLK/8, ACLK input
             TA0CCR0 = 150;         // ~1Hz       
             TA0EX0 |= TAIDEX_7;    // Divide input CLK/8
             TA0CTL |= MC_1;
          
             __bis_SR_register(LPM3_bits+GIE);       // Enter LPM0, enable interrupts
   

/*------------------------------------------------------------------------------
 Set ADC12_A 
------------------------------------------------------------------------------*/            
   
            REFCTL0 &= ~REFMSTR;             // Reset REFMSTR to hand over control to 
                                            // ADC12_A ref control registers
            ADC12CTL0 |= ADC12ON+ADC12MSC+ADC12SHT0_8+ADC12REFON; // ADC ON. Vref = 1.5V
            ADC12CTL2 |= ADC12REFOUT+ADC12REFBURST; // Turn on reference only during sample and conversion
            ADC12CTL1 |= ADC12SSEL_3+ADC12SHP+ADC12CONSEQ_3; //SMCLK. Sequence of channels
            ADC12MCTL0 |= ADC12SREF_1;      // Vr+=Vref+ and Vr-=AVss. Channel 0
            ADC12MCTL1 |= ADC12SREF_1;      // Vr+=Vref+ and Vr-=AVss. Channel 1
            for ( i=0; i<0x30; i++);                  // Delay for reference start-up
            ADC12CTL2 |= ADC12RES_2;  // 12 bit resolution
            ADC12MCTL0 |= ADC12INCH_0; // Channel 0 = TC
            ADC12MCTL1 |= ADC12INCH_1+ADC12EOS; // CHannel 1 = TR       
            
           
/*------------------------------------------------------------------------------
 Mainloop
------------------------------------------------------------------------------*/
 while(1){   
            Sample ();                // Measure TC and TR simultaneously
            P1OUT &= ~BIT0;
            Average ();               // Get average of 32 values
            Get_TR_Temp ();           // Lookup thermistor temp
            Get_TC_Temp ();           // Lookup thermocouple temp
            if(*Flags == Too_High || *Flags == Too_Low);// Test if either flag is set
            Error++;                  // Send too low or to high error message
            __bis_SR_register(LPM3_bits);               // LPM3
                
         }           
}
/*------------------------------------------------------------------------------
Interrupt Vectors
------------------------------------------------------------------------------*/

// Timer0 A0 interrupt service routine
#pragma vector=TIMER0_A0_VECTOR
__interrupt void TIMER0_A0_ISR(void)
{   
  P1OUT ^= BIT0;                          // For debuggind purposes
  __bic_SR_register_on_exit(LPM3_bits);   // Return Active
}


#pragma vector=ADC12_VECTOR
__interrupt void ADC12ISR (void)
{
  static unsigned int index = 0;

  switch(__even_in_range(ADC12IV,34))
  {
  case  0: break;                           // Vector  0:  No interrupt
  case  2: break;                           // Vector  2:  ADC overflow
  case  4: break;                           // Vector  4:  ADC timing overflow
  case  6: break;                           // Vector  6:  ADC12IFG0
  case  8:                                  // Vector  8:  ADC12IFG1
    A0results[index] = ADC12MEM0;           // Move A0 results, IFG is cleared
    A1results[index] = ADC12MEM1;           // Move A1 results, IFG is cleared
    index++;                                // Increment results index, modulo; 
   
    if (index == 32)
    {
      (index = 0);
       ADC12CTL0 &= ~ADC12ENC;              // Disable ADC
    }
  case 10: break;                           // Vector 10:  ADC12IFG2
  case 12: break;                           // Vector 12:  ADC12IFG3
  case 14: break;                           // Vector 14:  ADC12IFG4
  case 16: break;                           // Vector 16:  ADC12IFG5
  case 18: break;                           // Vector 18:  ADC12IFG6
  case 20: break;                           // Vector 20:  ADC12IFG7
  case 22: break;                           // Vector 22:  ADC12IFG8
  case 24: break;                           // Vector 24:  ADC12IFG9
  case 26: break;                           // Vector 26:  ADC12IFG10
  case 28: break;                           // Vector 28:  ADC12IFG11
  case 30: break;                           // Vector 30:  ADC12IFG12
  case 32: break;                           // Vector 32:  ADC12IFG13
  case 34: break;                           // Vector 34:  ADC12IFG14
  default: break; 
  }  
}

